@@ -62,7 +62,7 @@ dump_init(struct pci_access *a)
   struct pci_dev *dev = NULL;
   int len, mn, bn, dn, fn, i, j;
 
-  if (!a)
+  if (!name)
     a->error("dump: File name not given.");
   if (!(f = fopen(name, "r")))
     a->error("dump: Cannot open %s: %s", name, strerror(errno));
@@ -70,14 +70,18 @@ dump_init(struct pci_access *a)
     {
       char *z = strchr(buf, '\n');
       if (!z)
-	a->error("dump: line too long or unterminated");
+	{
+	  fclose(f);
+	  a->error("dump: line too long or unterminated");
+	}
       *z-- = 0;
       if (z >= buf && *z == '\r')
 	*z-- = 0;
       len = z - buf + 1;
       mn = 0;
       if (dump_validate(buf, "##:##.# ") && sscanf(buf, "%x:%x.%d", &bn, &dn, &fn) == 3 ||
-	  dump_validate(buf, "####:##:##.# ") && sscanf(buf, "%x:%x:%x.%d", &mn, &bn, &dn, &fn) == 4)
+	  dump_validate(buf, "####:##:##.# ") && sscanf(buf, "%x:%x:%x.%d", &mn, &bn, &dn, &fn) == 4 ||
+	  dump_validate(buf, "#####:##:##.# ") && sscanf(buf, "%x:%x:%x.%d", &mn, &bn, &dn, &fn) == 4)
 	{
 	  dev = pci_get_dev(a, mn, bn, dn, fn);
 	  dump_alloc_data(dev, 256);
@@ -86,7 +90,9 @@ dump_init(struct pci_access *a)
       else if (!len)
 	dev = NULL;
       else if (dev &&
-	       (dump_validate(buf, "##: ") || dump_validate(buf, "###: ")) &&
+	       (dump_validate(buf, "##: ") || dump_validate(buf, "###: ") || dump_validate(buf, "####: ") ||
+		dump_validate(buf, "#####: ") || dump_validate(buf, "######: ") ||
+		dump_validate(buf, "#######: ") || dump_validate(buf, "########: ")) &&
 	       sscanf(buf, "%x: ", &i) == 1)
 	{
 	  struct dump_data *dd = dev->aux;
@@ -95,7 +101,10 @@ dump_init(struct pci_access *a)
 		 sscanf(z, "%x", &j) == 1 && j < 256)
 	    {
 	      if (i >= 4096)
-		a->error("dump: At most 4096 bytes of config space are supported");
+		{
+		  fclose(f);
+		  a->error("dump: At most 4096 bytes of config space are supported");
+		}
 	      if (i >= dd->allocated)	/* Need to re-allocate the buffer */
 		{
 		  dump_alloc_data(dev, 4096);
@@ -111,9 +120,13 @@ dump_init(struct pci_access *a)
 		z++;
 	    }
 	  if (*z)
-	    a->error("dump: Malformed line");
+	    {
+	      fclose(f);
+	      a->error("dump: Malformed line");
+	    }
 	}
     }
+  fclose(f);
 }
 
 static void
@@ -173,6 +186,7 @@ struct pci_methods pm_dump = {
   pci_generic_fill_info,
   dump_read,
   dump_write,
+  NULL,					/* read_vpd */
   NULL,					/* init_dev */
   dump_cleanup_dev
 };
